package com.bwie.mq.config;

import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

/**
 * @CLassName MqConfig
 * @Description 描述
 * @Author Meng.Wang
 * @Date 2023/11/24 21:15
 */
@Configuration
@Log4j2
public class MqConfig implements RabbitTemplate.ReturnsCallback,RabbitTemplate.ConfirmCallback{
    public static final String DXQUEUE = "DxQueue";
    public static final String DXEXCHANGE = "DxExchange";
    public static final String ROUTINGKEY = "RoutingKey";

    private RabbitTemplate rabbitTemplate;

    //创建消息转换器
    @Bean
    public MessageConverter messageConverter(){
        return new Jackson2JsonMessageConverter();
    }

    //创建队列
    @Bean
    public Queue queue(){
        return new Queue(DXQUEUE,true);
    }
    //创建交换机
    @Primary
    @Bean
    public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory){
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        this.rabbitTemplate = rabbitTemplate;
        rabbitTemplate.setMessageConverter(messageConverter());
        rabbitTemplate();
        return rabbitTemplate;
    }

    @Bean("DxExchange")
    public DirectExchange directExchange(){
        return new DirectExchange(DXEXCHANGE);
    }

    public void rabbitTemplate(){
        rabbitTemplate.setConfirmCallback(this);
        rabbitTemplate.setReturnsCallback(this);
    }

    @Bean
    public Binding binding(){
        return BindingBuilder.bind(queue()).to(directExchange()).with(ROUTINGKEY);
    }

    @Override
    public void confirm(CorrelationData correlationData, boolean b, String s) {
        if(b){
            log.info("{}消息到达交换机",correlationData.getId());
        }else {
            log.error("{}消息丢失",correlationData.getId());
        }
    }

    @Override
    public void returnedMessage(ReturnedMessage returnedMessage) {
        log.error("{}消息未到达队列",returnedMessage.getMessage().getMessageProperties().getMessageId());
    }
}
    